import datetime
import typing
from enum import Enum
from typing import List, Union

from .aes import AES
from .banner import Banner, BannerColor
from .cosmetics import BrCosmetic, NewBrCosmetics, CarCosmetic, InstrumentCosmetic, LegoCosmeticVariant, JamTrack, \
    NewCosmetics, LegoKitCosmetic
from .creator_code import CreatorCode
from .enums import GameLanguage, MatchMethod, NewsType, KeyFormat, AccountType, TimeWindow, StatsImageType, \
    BrCosmeticRarity, BrCosmeticType
from .errors import MissingSearchParameter, MissingIDParameter, NotFound
from .map import Map
from .news import GameModeNews, News
from .playlist import Playlist
from .shop import BrShop, Shop
from .stats import BrPlayerStats

_SEARCH_PARAMETERS = {
    'language': [None, [GameLanguage]],
    'search_language': ['searchLanguage', [GameLanguage]],
    'match_method': ['matchMethod', [MatchMethod]],
    'id': [None, [str, None]],
    'name': [None, [str, None]],
    'description': [None, [str, None]],
    'type': [None, [BrCosmeticType, None]],
    'display_type': [None, [str, None]],
    'backend_type': ['backendType', [str, None]],
    'rarity': [None, [BrCosmeticRarity, None]],
    'display_rarity': ['displayRarity', [str, None]],
    'backend_rarity': ['backendRarity', [str, None]],
    'has_series': ['hasSeries', [bool, None]],
    'series': [None, [str, None]],
    'backend_series': ['backendSeries', [str, None]],
    'has_set': ['hasSet', [bool, None]],
    'set': ['set', [str, None]],
    'set_text': ['setText', [str, None]],
    'backend_set': ['backendSet', [str, None]],
    'has_introduction': ['hasIntroduction', [bool, None]],
    'backendIntroduction': ['backendIntroduction', [int, None]],
    'introduction_chapter': ['introductionChapter', [str, None]],
    'introduction_season': ['introductionSeason', [str, None]],
    'has_small_icon': ['hasSmallIcon', [bool, None]],
    'has_icon': ['hasIcon', [bool, None]],
    'has_featured_image': ['hasFeaturedImage', [bool, None]],
    'has_variants': ['hasVariants', [bool, None]],
    'has_gameplay_tags': ['hasGameplayTags', [bool, None]],
    'gameplay_tag': ['gameplayTag', [str, None]],
    'has_meta_tags': ['hasMetaTags', [bool, None]],
    'meta_tag': ['metaTag', [str, None]],
    'has_dynamic_pak_id': ['hasDynamicPakId', [bool, None]],
    'dynamic_pak_id': ['dynamicPakId', [str, None]],
    'added': [None, [datetime.datetime, None]],
    'added_since': ['addedSince', [datetime.datetime, None]],
    'unseen_for': ['unseenFor', [int, None]],
    'last_appearance': ['lastAppearance', [datetime.datetime, None]]
}


def _parse_search_parameter(**search_parameters):
    parameters = {}  # TODO: Empty string as search parameter
    for key, value in search_parameters.items():
        search_parameter_data = _SEARCH_PARAMETERS.get(key)
        if search_parameter_data is None:
            continue
        if type(value) not in search_parameter_data[1]:
            types = ' or '.join([str(t) for t in search_parameter_data[1]])
            raise TypeError('{0} require a value of type {1}'.format(key, types))
        key = search_parameter_data[0] if search_parameter_data[0] else key
        if value is None:
            value = '<null>'
        elif isinstance(value, Enum):
            value = value.value
        elif isinstance(value, datetime.datetime):
            value = int(value.timestamp())
        parameters[key] = value
    if len(parameters) == 0:
        raise MissingSearchParameter('at least one search parameter is required')
    return parameters


class SyncAESEndpoints:

    def __init__(self, client):
        self._client = client

    def fetch(self, key_format: KeyFormat = KeyFormat.HEX) -> AES:
        params = {'keyFormat': key_format.value}
        data = self._client.http.get('v2/aes', params=params)
        return AES(data['data'])


class AsyncAESEndpoints:

    def __init__(self, client):
        self._client = client

    async def fetch(self, key_format: KeyFormat = KeyFormat.HEX) -> AES:
        params = {'keyFormat': key_format.value}
        data = await self._client.http.get('v2/aes', params=params)
        return AES(data['data'])


class SyncBannerEndpoints:

    def __init__(self, client):
        self._client = client

    def fetch(self, language: GameLanguage = GameLanguage.ENGLISH) -> typing.List[Banner]:
        params = {'language': language.value}
        data = self._client.http.get('v1/banners', params=params)
        return [Banner(item_data) for item_data in data['data']]

    def fetch_colors(self) -> typing.List[BannerColor]:
        data = self._client.http.get('v1/banners/colors')
        return [BannerColor(item_data) for item_data in data['data']]


class AsyncBannerEndpoints:

    def __init__(self, client):
        self._client = client

    async def fetch(self, language: GameLanguage = GameLanguage.ENGLISH) -> typing.List[Banner]:
        params = {'language': language.value}
        data = await self._client.http.get('v1/banners', params=params)
        return [Banner(item_data) for item_data in data['data']]

    async def fetch_colors(self) -> typing.List[BannerColor]:
        data = await self._client.http.get('v1/banners/colors')
        return [BannerColor(item_data) for item_data in data['data']]


class SyncCosmeticsEndpoints:

    def __init__(self, client):
        self._client = client

    def fetch(
            self,
            language=GameLanguage.ENGLISH
    ) -> List[Union[BrCosmetic, CarCosmetic, InstrumentCosmetic, LegoKitCosmetic, JamTrack, LegoCosmeticVariant]]:
        params = {'language': language.value}
        data = self._client.http.get('v2/cosmetics', params=params)
        data = data['data']
        cosmetics = []
        cosmetics.extend([BrCosmetic(item) for item in data['br']])
        cosmetics.extend([CarCosmetic(item) for item in data['cars']])
        cosmetics.extend([InstrumentCosmetic(item) for item in data['instruments']])
        cosmetics.extend([LegoKitCosmetic(item) for item in data['legoKits']])
        cosmetics.extend([JamTrack(item) for item in data['tracks']])
        cosmetics.extend([LegoCosmeticVariant(item) for item in data['lego']])
        return cosmetics

    def fetch_br(self, language=GameLanguage.ENGLISH):
        """Returns an list of all battle royale cosmetics.

        Parameters
        -----------
        language: :class:`GameLanguage`
            The language in which the cosmetics should be returned. Default is set to english.

        Returns
        --------
        List[:class:`BrCosmetic`]
            A list of all cosmetics.

        Raises
        -------
        Unauthorized
            You have not used a valid API key.
        RateLimited
            You reached the rate limit.
        ServiceUnavailable
            The Fortnite-API.com server is currently not available.
        """
        params = {'language': language.value}
        data = self._client.http.get('v2/cosmetics/br', params=params)
        return [BrCosmetic(item_data) for item_data in data['data']]

    fetch_all = fetch_br  # Backwards compatibility

    def fetch_cars(self, language=GameLanguage.ENGLISH):
        params = {'language': language.value}
        data = self._client.http.get('v2/cosmetics/cars', params=params)
        return [CarCosmetic(item_data) for item_data in data['data']]

    def fetch_instruments(self, language=GameLanguage.ENGLISH):
        params = {'language': language.value}
        data = self._client.http.get('v2/cosmetics/instruments', params=params)
        return [InstrumentCosmetic(item_data) for item_data in data['data']]

    def fetch_lego_kits(self, language=GameLanguage.ENGLISH):
        params = {'language': language.value}
        data = self._client.http.get('v2/cosmetics/lego/kits', params=params)
        return [LegoKitCosmetic(item_data) for item_data in data['data']]

    def fetch_jam_tracks(self, language=GameLanguage.ENGLISH):
        params = {'language': language.value}
        data = self._client.http.get('v2/cosmetics/tracks', params=params)
        return [JamTrack(item_data) for item_data in data['data']]

    def fetch_lego_variants(self, language=GameLanguage.ENGLISH):
        params = {'language': language.value}
        data = self._client.http.get('v2/cosmetics/lego', params=params)
        return [LegoCosmeticVariant(item_data) for item_data in data['data']]

    def fetch_new_all(self, language=GameLanguage.ENGLISH):
        params = {'language': language.value}
        data = self._client.http.get('v2/cosmetics/new', params=params)
        return NewCosmetics(data['data'])

    def fetch_new_br(self, language=GameLanguage.ENGLISH):
        params = {'language': language.value}
        data = self._client.http.get('v2/cosmetics/br/new', params=params)
        return NewBrCosmetics(data['data'])

    fetch_new = fetch_new_br  # Backwards compatibility

    def search_by_id(self, *cosmetic_ids, language=GameLanguage.ENGLISH):
        """Returns an list of the requested battle royale cosmetic ids.

        Parameters
        -----------
        \*cosmetic_ids: :class:`str`
            The id of the cosmetic

        Returns
        --------
        List[:class:`BrCosmetic`]
            A list of all cosmetics.

        Raises
        -------
        Unauthorized
            You have not used a valid API key.
        RateLimited
            You reached the rate limit.
        ServiceUnavailable
            The Fortnite-API.com server is currently not available.
        """
        cosmetic_ids = list(cosmetic_ids)
        params = {'language': language.value}

        if len(cosmetic_ids) == 0:
            raise MissingIDParameter('at least one cosmetic id is required')

        endpoint = 'v2/cosmetics/br/search/ids'
        endpoint += '?id=' + '&id='.join(cosmetic_ids)
        data = self._client.http.get(endpoint, params=params)
        return [BrCosmetic(item_data) for item_data in data['data']]

    def search_all(self, **search_parameters) -> typing.List[BrCosmetic]:
        data = self._client.http.get('v2/cosmetics/br/search/all',
                                     params=_parse_search_parameter(**search_parameters))
        return [BrCosmetic(item_data) for item_data in data['data']]

    def search_first(self, **search_parameters) -> BrCosmetic:
        data = self._client.http.get('v2/cosmetics/br/search',
                                     params=_parse_search_parameter(**search_parameters))
        return BrCosmetic(data['data'])


class AsyncCosmeticsEndpoints:

    def __init__(self, client):
        self._client = client

    async def fetch(
            self,
            language=GameLanguage.ENGLISH
    ) -> List[Union[BrCosmetic, CarCosmetic, InstrumentCosmetic, LegoKitCosmetic, JamTrack, LegoCosmeticVariant]]:
        params = {'language': language.value}
        data = await self._client.http.get('v2/cosmetics', params=params)
        data = data['data']
        cosmetics = []
        cosmetics.extend([BrCosmetic(item) for item in data['br']])
        cosmetics.extend([CarCosmetic(item) for item in data['cars']])
        cosmetics.extend([InstrumentCosmetic(item) for item in data['instruments']])
        cosmetics.extend([LegoKitCosmetic(item) for item in data['legoKits']])
        cosmetics.extend([JamTrack(item) for item in data['tracks']])
        cosmetics.extend([LegoCosmeticVariant(item) for item in data['lego']])
        return cosmetics

    async def fetch_br(self, language: GameLanguage = GameLanguage.ENGLISH) -> typing.List[BrCosmetic]:
        params = {'language': language.value}
        data = await self._client.http.get('v2/cosmetics/br', params=params)
        return [BrCosmetic(item_data) for item_data in data['data']]

    fetch_all = fetch_br  # Backwards compatibility

    async def fetch_cars(self, language=GameLanguage.ENGLISH):
        params = {'language': language.value}
        data = await self._client.http.get('v2/cosmetics/cars', params=params)
        return [CarCosmetic(item_data) for item_data in data['data']]

    async def fetch_instruments(self, language=GameLanguage.ENGLISH):
        params = {'language': language.value}
        data = await self._client.http.get('v2/cosmetics/instruments', params=params)
        return [InstrumentCosmetic(item_data) for item_data in data['data']]

    async def fetch_lego_kits(self, language=GameLanguage.ENGLISH):
        params = {'language': language.value}
        data = await self._client.http.get('v2/cosmetics/lego/kits', params=params)
        return [LegoKitCosmetic(item_data) for item_data in data['data']]

    async def fetch_jam_tracks(self, language=GameLanguage.ENGLISH):
        params = {'language': language.value}
        data = await self._client.http.get('v2/cosmetics/tracks', params=params)
        return [JamTrack(item_data) for item_data in data['data']]

    async def fetch_lego_variants(self, language=GameLanguage.ENGLISH):
        params = {'language': language.value}
        data = await self._client.http.get('v2/cosmetics/lego', params=params)
        return [LegoCosmeticVariant(item_data) for item_data in data['data']]

    async def fetch_new_all(self, language=GameLanguage.ENGLISH):
        params = {'language': language.value}
        data = await self._client.http.get('v2/cosmetics/new', params=params)
        return NewCosmetics(data['data'])

    async def fetch_new_br(self, language=GameLanguage.ENGLISH):
        params = {'language': language.value}
        data = await self._client.http.get('v2/cosmetics/br/new', params=params)
        return NewBrCosmetics(data['data'])

    fetch_new = fetch_new_br  # Backwards compatibility

    async def search_by_id(self, *cosmetic_id: str, language: GameLanguage = GameLanguage.ENGLISH) -> typing.List[
        BrCosmetic]:
        cosmetic_ids = list(cosmetic_id)
        params = {'language': language.value}

        if len(cosmetic_ids) == 0:
            raise MissingIDParameter('at least one cosmetic id is required')

        endpoint = 'v2/cosmetics/br/search/ids'
        endpoint += '?id=' + '&id='.join(cosmetic_ids)
        data = await self._client.http.get(endpoint, params=params)
        return [BrCosmetic(item_data) for item_data in data['data']]

    async def search_all(self, **search_parameters) -> typing.List[BrCosmetic]:
        data = await self._client.http.get('v2/cosmetics/br/search/all',
                                           params=_parse_search_parameter(**search_parameters))
        return [BrCosmetic(item_data) for item_data in data['data']]

    async def search_first(self, **search_parameters) -> BrCosmetic:
        data = await self._client.http.get('v2/cosmetics/br/search',
                                           params=_parse_search_parameter(**search_parameters))
        return BrCosmetic(data['data'])


class SyncCreatorCodeEndpoints:

    def __init__(self, client):
        self._client = client

    def fetch(self, name: str) -> CreatorCode:
        params = {'name': name}
        data = self._client.http.get('v2/creatorcode', params=params)
        return CreatorCode(data['data'])

    def exists(self, name: str) -> bool:
        try:
            self.fetch(name)
            return True
        except NotFound:
            return False

    def search_first(self, name: str) -> CreatorCode:
        params = {'name': name}
        data = self._client.http.get('v2/creatorcode/search', params=params)
        return CreatorCode(data['data'])

    def search_all(self, name: str) -> typing.List[CreatorCode]:
        params = {'name': name}
        data = self._client.http.get('v2/creatorcode/search/all', params=params)
        return [CreatorCode(creator_code_data) for creator_code_data in data['data']]


class AsyncCreatorCodeEndpoints:

    def __init__(self, client):
        self._client = client

    async def fetch(self, name: str) -> CreatorCode:
        params = {'name': name}
        data = await self._client.http.get('v2/creatorcode', params=params)
        return CreatorCode(data['data'])

    async def exists(self, name: str) -> bool:
        try:
            await self.fetch(name)
            return True
        except NotFound:
            return False

    async def search_first(self, name: str) -> CreatorCode:
        params = {'name': name}
        data = await self._client.http.get('v2/creatorcode/search', params=params)
        return CreatorCode(data['data'])

    async def search_all(self, name: str) -> typing.List[CreatorCode]:
        params = {'name': name}
        data = await self._client.http.get('v2/creatorcode/search/all', params=params)
        return [CreatorCode(creator_code_data) for creator_code_data in data['data']]


class SyncMapEndpoints:

    def __init__(self, client):
        self._client = client

    def fetch(self, language: GameLanguage = GameLanguage.ENGLISH) -> Map:
        params = {'language': language.value}
        data = self._client.http.get('v1/map', params=params)
        return Map(data['data'])


class AsyncMapEndpoints:

    def __init__(self, client):
        self._client = client

    async def fetch(self, language: GameLanguage = GameLanguage.ENGLISH) -> Map:
        params = {'language': language.value}
        data = await self._client.http.get('v1/map', params=params)
        return Map(data['data'])


class SyncNewsEndpoints:

    def __init__(self, client):
        self._client = client

    def fetch(self, language: GameLanguage = GameLanguage.ENGLISH) -> News:
        params = {'language': language.value}
        data = self._client.http.get('v2/news', params=params)
        return News(data['data'])

    def fetch_by_type(self, news_type: NewsType, language=GameLanguage.ENGLISH) -> GameModeNews:
        params = {'language': language.value}
        data = self._client.http.get('v2/news/' + news_type.value, params=params)
        return GameModeNews(data['data'])


class AsyncNewsEndpoints:

    def __init__(self, client):
        self._client = client

    async def fetch(self, language: GameLanguage = GameLanguage.ENGLISH) -> News:
        params = {'language': language.value}
        data = await self._client.http.get('v2/news', params=params)
        return News(data['data'])

    async def fetch_by_type(self, news_type: NewsType, language: GameLanguage = GameLanguage.ENGLISH) -> GameModeNews:
        params = {'language': language.value}
        data = await self._client.http.get('v2/news/' + news_type.value, params=params)
        return GameModeNews(data['data'])


class SyncPlaylistEndpoints:

    def __init__(self, client):
        self._client = client

    def fetch_all(self, language: GameLanguage = GameLanguage.ENGLISH) -> List[Playlist]:
        params = {'language': language.value}
        data = self._client.http.get('v1/playlists', params=params)
        return [Playlist(playlist) for playlist in data['data']]

    def fetch_by_id(self, playlist_id, language: GameLanguage = GameLanguage.ENGLISH) -> Playlist:
        params = {'language': language.value}
        data = self._client.http.get(f'v1/playlists/{playlist_id}', params=params)
        return Playlist(data['data'])


class AsyncPlaylistEndpoints:

    def __init__(self, client):
        self._client = client

    async def fetch_all(self, language: GameLanguage = GameLanguage.ENGLISH) -> List[Playlist]:
        params = {'language': language.value}
        data = await self._client.http.get('v1/playlists', params=params)
        return [Playlist(playlist) for playlist in data['data']]

    async def fetch_by_id(self, playlist_id, language: GameLanguage = GameLanguage.ENGLISH) -> Playlist:
        params = {'language': language.value}
        data = await self._client.http.get(f'v1/playlists/{playlist_id}', params=params)
        return Playlist(data['data'])


class SyncShopEndpoints:

    def __init__(self, client):
        self._client = client

    def fetch_all(self, language: GameLanguage = GameLanguage.ENGLISH) -> Shop:
        params = {'language': language.value}
        data = self._client.http.get('v2/shop', params=params)
        return Shop(data['data'])

    def fetch_br(self, language: GameLanguage = GameLanguage.ENGLISH, combined: bool = False) -> BrShop:
        params = {'language': language.value}
        data = self._client.http.get('v2/shop/br' if not combined else 'v2/shop/br/combined', params=params)
        return BrShop(data['data'])

    fetch = fetch_br


class AsyncShopEndpoints:

    def __init__(self, client):
        self._client = client

    async def fetch_all(self, language: GameLanguage = GameLanguage.ENGLISH) -> Shop:
        params = {'language': language.value}
        data = await self._client.http.get('v2/shop', params=params)
        return Shop(data['data'])

    async def fetch_br(self, language: GameLanguage = GameLanguage.ENGLISH, combined: bool = False) -> BrShop:
        params = {'language': language.value}
        data = await self._client.http.get('v2/shop/br' if not combined else 'v2/shop/br/combined', params=params)
        return BrShop(data['data'])

    fetch = fetch_br


class SyncStatsEndpoints:

    def __init__(self, client):
        self._client = client

    def fetch_by_name(self, name: str,
                      account_type: AccountType = AccountType.EPIC,
                      time_window: TimeWindow = TimeWindow.LIFETIME,
                      image: StatsImageType = StatsImageType.NONE) -> BrPlayerStats:
        params = {'name': name, 'accountType': account_type.value, 'timeWindow': time_window.value,
                  'image': image.value}
        data = self._client.http.get('v2/stats/br/v2', params=params)
        return BrPlayerStats(data['data'])

    def fetch_by_id(self, account_id: str,
                    time_window: TimeWindow = TimeWindow.LIFETIME,
                    image: StatsImageType = StatsImageType.NONE) -> BrPlayerStats:
        params = {'timeWindow': time_window.value, 'image': image.value}
        data = self._client.http.get('v2/stats/br/v2/{account_id}'.format(account_id=account_id), params=params)
        return BrPlayerStats(data['data'])


class AsyncStatsEndpoints:

    def __init__(self, client):
        self._client = client

    async def fetch_by_name(self, name: str,
                            account_type: AccountType = AccountType.EPIC,
                            time_window: TimeWindow = TimeWindow.LIFETIME,
                            image: StatsImageType = StatsImageType.NONE) -> BrPlayerStats:
        params = {'name': name, 'accountType': account_type.value, 'timeWindow': time_window.value,
                  'image': image.value}
        data = await self._client.http.get('v2/stats/br/v2', params=params)
        return BrPlayerStats(data['data'])

    async def fetch_by_id(self, account_id: str,
                          time_window: TimeWindow = TimeWindow.LIFETIME,
                          image: StatsImageType = StatsImageType.NONE) -> BrPlayerStats:
        params = {'timeWindow': time_window.value, 'image': image.value}
        data = await self._client.http.get('v2/stats/br/v2/{account_id}'.format(account_id=account_id), params=params)
        return BrPlayerStats(data['data'])



from datetime import datetime
from typing import List, Union

from fortnite_api.enums import BrCosmeticType, BrCosmeticRarity


class NewBrCosmetics:

    def __init__(self, data):
        self.build = data.get('build')
        self.previous_build = data.get('previousBuild')
        self.hash = data.get('hash')
        try:
            self.date = datetime.strptime(data.get('date'), '%Y-%m-%dT%H:%M:%S%z')
        except (ValueError, TypeError):
            self.date = None
        try:
            self.last_addition = datetime.strptime(data.get('lastAddition'), '%Y-%m-%dT%H:%M:%S%z')
        except (ValueError, TypeError):
            self.last_addition = None
        self.items = [BrCosmetic(i) for i in data.get('items')] if data.get('items') else None
        self.raw_data = data


class NewCosmetics:

    def __init__(self, data):
        self.date: datetime = datetime.strptime(data['date'], '%Y-%m-%dT%H:%M:%S%z')
        self.build: str = data['build']
        self.previous_build: str = data['previousBuild']
        hashes = data['hashes']
        self.hash: str = hashes['all']
        self.hash_br: str = hashes['br']
        self.hash_cars: str = hashes['cars']
        self.hash_instruments: str = hashes['instruments']
        self.hash_jam_tracks: str = hashes['tracks']
        self.hash_lego: str = hashes['lego']
        last_additions = data['lastAdditions']
        self.last_addition: datetime = datetime.strptime(last_additions['all'], '%Y-%m-%dT%H:%M:%S%z')
        self.last_addition_br: datetime = datetime.strptime(last_additions['br'], '%Y-%m-%dT%H:%M:%S%z')
        self.last_addition_cars: datetime = datetime.strptime(last_additions['cars'], '%Y-%m-%dT%H:%M:%S%z')
        self.last_addition_instruments: datetime = datetime.strptime(
            last_additions['instruments'], '%Y-%m-%dT%H:%M:%S%z'
        )
        self.last_addition_jam_tracks: datetime = datetime.strptime(last_additions['tracks'], '%Y-%m-%dT%H:%M:%S%z')
        self.last_addition_lego: datetime = datetime.strptime(last_additions['lego'], '%Y-%m-%dT%H:%M:%S%z')
        items = data['items']
        self.items: List[Union[BrCosmetic, CarCosmetic, InstrumentCosmetic, LegoKitCosmetic, JamTrack, LegoCosmeticVariant]] = []
        self.items.extend([BrCosmetic(item) for item in items['br']])
        self.items.extend([CarCosmetic(item) for item in items['cars']])
        self.items.extend([InstrumentCosmetic(item) for item in items['instruments']])
        self.items.extend([LegoKitCosmetic(item) for item in items['legoKits']])
        self.items.extend([JamTrack(item) for item in items['tracks']])
        self.items.extend([LegoCosmeticVariant(item) for item in items['lego']])


class BaseCosmetic:
    """Represents a Cosmetic.

    Attributes
    -----------
    id: :class:`str`
        The id of the cosmetic.
    type: :class:`BrCosmeticType`
        The type of the cosmetic.
    backend_type: :class:`str`
        The internal type of the cosmetic.
    rarity: :class:`BrCosmeticRarity`
        The rarity of the cosmetic.
    backend_rarity: :class:`str`
        The internal rarity of the cosmetic.
    name: :class:`str`
        The name of the cosmetic in the chosen language.
    description: :class:`str`
        The description of the cosmetic in the chosen language.
    series: Optional[:class:`str`]
        The series of the cosmetic in the chosen language.
    backend_series: Optional[:class:`str`]
        The internal series of the cosmetic.
    gameplay_tags: Optional[List[:class:`str`]]
        A :class:`list` of gameplay tags of the cosmetics.
    path: :class:`str`
        The path of the asset.
    added: :class:`datetime.datetime`
        The timestamp when the item was added to the Fortnite-API.com database.
    raw_data: :class:`dict`
        The raw data from request. Can be used for saving and recreating the class.
    """

    def __init__(self, data):
        self.id = data.get('id')
        self.name = data.get('name')
        self.description = data.get('description')

        cosmetic_type = data.get('type', {}) if data.get('type') else {}
        try:
            self.type = BrCosmeticType(cosmetic_type.get('value'))
        except ValueError:
            self.type = BrCosmeticType.UNKNOWN
        self.display_type = cosmetic_type.get('displayValue')
        self.backend_type = cosmetic_type.get('backendValue')

        rarity = data.get('rarity', {}) if data.get('rarity') else {}
        try:
            self.rarity = BrCosmeticRarity(rarity.get('value'))
        except ValueError:
            self.rarity = BrCosmeticRarity.UNKNOWN
        self.rarity_text = rarity.get('displayValue')
        self.backend_rarity = rarity.get('backendValue')

        series = data.get('series', {}) if data.get('series') else {}
        self.series = series.get('value')
        self.series_image = series.get('image')
        self.series_colors = series.get('colors')
        self.backend_series = series.get('backendValue')

        self.gameplay_tags = [gameplay_tag for gameplay_tag in data.get('gameplayTags')] \
            if data.get('gameplayTags') is not None else None
        self.showcase_video_url = 'https://youtube.com/watch?v=' + data.get('showcaseVideo') \
            if data.get('showcaseVideo') else None
        self.path = data.get('path')
        try:
            self.added = datetime.strptime(data.get('added'), '%Y-%m-%dT%H:%M:%S%z')
        except (ValueError, TypeError):
            self.added = None

        self.shop_history = []
        for date in data.get('shopHistory', []) if data.get('shopHistory') else []:
            try:
                self.shop_history.append(datetime.strptime(date, '%Y-%m-%dT%H:%M:%S%z').replace(tzinfo=None))
            except (ValueError, TypeError):
                pass
        self.appearances = len(self.shop_history)
        self.first_appearance = self.shop_history[0] if self.appearances > 0 else None
        self.last_appearance = self.shop_history[self.appearances - 1] if self.appearances > 0 else None
        self.unseen_for = (datetime.utcnow() - self.last_appearance).days if self.last_appearance else None

        self.raw_data = data


class BrCosmetic(BaseCosmetic):
    """Represents a Battle Royale Cosmetic.

    Attributes
    -----------
    set: Optional[:class:`str`]
        The set of the cosmetic in the chosen language.
    set_text: Optional[:class:`str`]
        The text of the set of the cosmetic in the chosen language.
    small_icon: :class:`BrCosmeticImage`
        The icon image in 128x128 resolution of the cosmetic.
    icon: Optional[:class:`BrCosmeticImage`]
        The icon image in 512x512 resolution of the cosmetic.
    featured: Optional[:class:`BrCosmeticImage`]
        The featured image in 1024x1024 resolution of the cosmetic.
    background: Optional[:class:`BrCosmeticImage`]
        The background image in 2048x1024 resolution of a loading screen.
    coverart: Optional[:class:`BrCosmeticImage`]
        The cover art image in 512x512 resolution of a music pack.
    decal: Optional[:class:`BrCosmeticImage`]
        The decal in 512x512 resolution of a spray.
    variants: Optional[List[:class:`BrCosmeticVariant`]]
        A :class:`list` of :class:`BrCosmeticVariant` of the cosmetic.
    display_asset_path: Optional[:class:`str`]
        The path of the display asset.
    """

    def __init__(self, data):
        super().__init__(data)
        self.exclusive_description = data.get('exclusiveDescription')
        self.unlock_requirements = data.get('unlockRequirements')
        self.custom_exclusive_callout = data.get('customExclusiveCallout')

        cosmetic_set = data.get('set', {}) if data.get('set') else {}
        self.set = cosmetic_set.get('value')
        self.set_text = cosmetic_set.get('text')
        self.backend_set = cosmetic_set.get('backendValue')

        introduction = data.get('introduction', {}) if data.get('introduction') else {}
        self.introduction_chapter = introduction.get('chapter')
        self.introduction_season = introduction.get('season')
        self.introduction_text = introduction.get('text')
        self.backend_introduction = introduction.get('backendValue')

        images = data.get('images', {}) if data.get('images') else {}
        self.small_icon = BrCosmeticImage(images.get('smallIcon')) if images.get('smallIcon') else None
        self.icon = BrCosmeticImage(images.get('icon')) if images.get('icon') else None
        self.featured = BrCosmeticImage(images.get('featured')) if images.get('featured') else None
        lego_images = images.get('lego', {}) if images.get('lego') else {}
        self.lego_small = BrCosmeticImage(lego_images.get('small')) if lego_images.get('small') else None
        self.lego_large = BrCosmeticImage(lego_images.get('large')) if lego_images.get('large') else None
        self.lego_wide = BrCosmeticImage(lego_images.get('wide')) if lego_images.get('wide') else None
        other_images = images.get('other', {}) if images.get('other') else {}
        self.background = BrCosmeticImage(other_images.get('background')) if other_images.get('background') else None
        self.coverart = BrCosmeticImage(other_images.get('coverart')) if other_images.get('coverart') else None
        self.decal = BrCosmeticImage(other_images.get('decal')) if other_images.get('decal') else None

        self.variants = [BrCosmeticVariant(variant) for variant in data.get('variants')] \
            if data.get('variants') is not None else None
        self.built_in_emote_ids = [be for be in data.get('builtInEmoteIds')] \
            if data.get('builtInEmoteIds') is not None else None
        self.search_tags = [st for st in data.get('searchTags')] \
            if data.get('searchTags') is not None else None

        self.meta_tags = [meta_tag for meta_tag in data.get('metaTags')] \
            if data.get('metaTags') is not None else None

        self.dynamic_pak_id = data.get('dynamicPakId')
        self.item_preview_hero_path = data.get('itemPreviewHeroPath')
        self.display_asset_path = data.get('displayAssetPath')
        self.definition_path = data.get('definitionPath')


class BrCosmeticVariant:
    """Represents a Battle Royale cosmetic image.

    Attributes
    -----------
    channel: :class:`str`
        The channel of the variant.
    type: Optional[:class:`str`]
        The type of the variant in the chosen language.
    options: List[:class:`BrCosmeticVariantOption`]
        A :class:`list` of :class:`BrCosmeticVariantOption` of the variant.
    raw_data: :class:`dict`
        The raw data from request. Can be used for saving and re-creating the class.
    """

    def __init__(self, data):
        self.channel = data.get('channel')
        self.type = data.get('type')
        self.options = [BrCosmeticVariantOption(option) for option in data.get('options')] \
            if data.get('options') is not None else None
        self.raw_data = data


class BrCosmeticVariantOption:
    """Represents a Battle Royale cosmetic image.

    Attributes
    -----------
    tag: :class:`str`
        The tag of the option.
    name: :class:`str`
        The name of the option in the chosen language.
    image: :class:`BrCosmeticImage`
        A image of the option.
    raw_data: :class:`dict`
        The raw data from request. Can be used for saving and re-creating the class.
    """

    def __init__(self, data):
        self.tag = data.get('tag')
        self.name = data.get('name')
        self.image = BrCosmeticImage(data.get('image'))
        self.unlock_requirements = data.get('unlockRequirements')
        self.raw_data = data


class CarCosmetic(BaseCosmetic):
    """Represents a Car Cosmetic.

    Attributes
    -----------
    vehicle_id: :class:`str`
        The id of the vehicle.
    small: :class:`CosmeticImage`
        The small image of the cosmetic.
    large: :class:`CosmeticImage`
        The large image of the cosmetic.
    """

    def __init__(self, data):
        super().__init__(data)
        self.vehicle_id = data.get('vehicleId')

        images = data.get('images', {}) if data.get('images') else {}
        self.small = CosmeticImage(images.get('small')) if images.get('small') else None
        self.large = CosmeticImage(images.get('large')) if images.get('large') else None


class InstrumentCosmetic(BaseCosmetic):
    """Represents an Instrument Cosmetic.

    Attributes
    -----------
    small: :class:`CosmeticImage`
        The small image of the cosmetic.
    large: :class:`CosmeticImage`
        The large image of the cosmetic.
    """

    def __init__(self, data):
        super().__init__(data)
        images = data.get('images', {}) if data.get('images') else {}
        self.small = CosmeticImage(images.get('small')) if images.get('small') else None
        self.large = CosmeticImage(images.get('large')) if images.get('large') else None


class LegoKitCosmetic(BaseCosmetic):

    def __init__(self, data):
        super().__init__(data)
        images = data.get('images', {}) if data.get('images') else {}
        self.small = CosmeticImage(images.get('small')) if images.get('small') else None
        self.large = CosmeticImage(images.get('large')) if images.get('large') else None


class LegoCosmeticVariant:
    """Represents a Lego Cosmetic Variant.

    Attributes
    -----------
    tag: :class:`str`
        The tag of the variant.
    name: :class:`str`
        The name of the variant in the chosen language.
    image: :class:`BrCosmeticImage`
        A image of the variant.
    raw_data: :class:`dict`
        The raw data from request. Can be used for saving and re-creating the class.
    """

    def __init__(self, data):
        self.cosmetic_id = data.get('cosmeticId')
        self.sound_library_tags = data.get('soundLibraryTags')
        images = data.get('images', {}) if data.get('images') else {}
        self.small = CosmeticImage(images.get('small')) if images.get('small') else None
        self.large = CosmeticImage(images.get('large')) if images.get('large') else None
        self.wide = CosmeticImage(images.get('wide')) if images.get('wide') else None
        self.path = data.get('path')


class JamTrack:
    """Represents a Jam Track.

    Attributes
    -----------
    id: :class:`str`
        The id of the jam track.
    dev_name: :class:`str`
        The dev name of the jam track.
    title: :class:`str`
        The title of the jam track.
    artist: :class:`str`
        The artist of the jam track.
    album: :class:`str`
        The album of the jam track.
    release_year: :class:`int`
        The release year of the jam track.
    bpm: :class:`int`
        The bpm of the jam track.
    duration: :class:`int`
        The duration of the jam track.
    difficutly: :class:`int`
        The difficulty of the jam track.
    gameplay_tags: List[:class:`str`]
        A :class:`list` of gameplay tags of the jam track.
    genres: List[:class:`str`]
        A :class:`list` of genres of the jam track.
    album_art_url: :class:`str`
        The album art url of the jam track.
    added: :class:`datetime.datetime`
        The timestamp when the item was added to the Fortnite-API.com database.
    shop_history: List[:class:`datetime.datetime`]
        A :class:`list` of timestamps when the jam track was in the shop.
    appearances: :class:`int`
        The amount of times the jam track was in the shop.
    first_appearance: Optional[:class:`datetime.datetime`]
        The timestamp when the jam track was first in the shop.
    last_appearance: Optional[:class:`datetime.datetime`]
        The timestamp when the jam track was last in the shop.
    unseen_for: Optional[:class:`int`]
        The amount of days the jam track is unseen.
    raw_data: :class:`dict`
        The raw data from request. Can be used for saving and re-creating the class.
    """

    def __init__(self, data):
        self.id = data.get('id')
        self.dev_name = data.get('devName')
        self.title = data.get('title')
        self.artist = data.get('artist')
        self.album = data.get('album')
        self.release_year = data.get('releaseYear')
        self.bpm = data.get('bpm')
        self.duration = data.get('duration')
        self.difficutly = data.get('difficutly')
        self.gameplay_tags = data.get('gameplayTags')
        self.genres = data.get('genres')
        self.album_art_url = data.get('albumArt')
        try:
            self.added = datetime.strptime(data.get('added'), '%Y-%m-%dT%H:%M:%S%z')
        except (ValueError, TypeError):
            self.added = None
        self.shop_history = []
        for date in data.get('shopHistory', []) if data.get('shopHistory') else []:
            try:
                self.shop_history.append(datetime.strptime(date, '%Y-%m-%dT%H:%M:%S%z').replace(tzinfo=None))
            except (ValueError, TypeError):
                pass
        self.appearances = len(self.shop_history)
        self.first_appearance = self.shop_history[0] if self.appearances > 0 else None
        self.last_appearance = self.shop_history[self.appearances - 1] if self.appearances > 0 else None
        self.unseen_for = (datetime.utcnow() - self.last_appearance).days if self.last_appearance else None
        self.raw_data = data


class CosmeticImage:
    """Represents a Battle Royale cosmetic image.

    Attributes
    -----------
    url: :class:`str`
        The hash of the image.
    """

    def __init__(self, url):
        self.url = url

    def url_as(self, size):
        if (size & (size - 1) != 0) or size <= 0:
            raise ValueError('Size must be a positive power of 2')

        url_without_type = self.url.replace('.png', '')
        return url_without_type + '_' + str(size) + '.png'

    def __str__(self):
        return self.url


BrCosmeticImage = CosmeticImage



import math


class Banner:

    def __init__(self, data):
        self.id = data.get('id')
        self.name = data.get('name')
        self.description = data.get('description')
        self.category = data.get('category')
        self.full_usage_rights = data.get('fullUsageRights')

        images = data.get('images', {}) if data.get('images') else {}
        self.small_icon = BrBannerImage(images.get('smallIcon')) if images.get('smallIcon') else None
        self.icon = BrBannerImage(images.get('icon')) if images.get('icon') else None
        self.raw_data = data


class BannerColor:

    def __init__(self, data):
        self.id = data.get('id')
        self.color = data.get('color')
        self.category = data.get('category')
        self.sub_category_group = data.get('subCategoryGroup')
        self.raw_data = data


class BrBannerImage:
    """Represents a Battle Royale cosmetic image.

    Attributes
    -----------
    url: :class:`str`
        The hash of the image.
    """

    def __init__(self, url):
        self.url = url

    def url_as(self, size):
        if (size & (size - 1) != 0) or size <= 0:
            raise ValueError('Size must be a positive power of 2')
        url_without_type = self.url.replace('.png', '')
        return url_without_type + '_' + str(size) + '.png'

    def __str__(self):
        return self.url



from datetime import datetime


class News:
    """Represents Game News.

    Attributes
    -----------
    br: List[:class:`GameModeNews`]
        A list of Battle Royale news.
    stw: List[:class:`GameModeNews`]
        A list of Save the World news.
    creative: List[:class:`GameModeNews`]
        A list of Creative news.
    raw_data: :class:`dict`
        The raw data from request. Can be used for saving and re-creating the class.
    """

    def __init__(self, data):
        self.br = GameModeNews(data.get('br')) if data.get('br') else None
        self.stw = GameModeNews(data.get('stw')) if data.get('stw') else None
        self.creative = GameModeNews(data.get('creative')) if data.get('creative') else None
        self.raw_data = data


class GameModeNews:
    """Represents News of a specific game mode.

    Attributes
    -----------
    last_modified: :class:`datetime.datetime`
        The timestamp when the news where modified.
    entries: List[:class:`NewsEntry`]
        A list of news entries.
    raw_data: :class:`dict`
        The raw data from request. Can be used for saving and re-creating the class.
    """

    def __init__(self, data):
        self.hash = data.get('hash')
        try:
            self.date = datetime.strptime(data.get('date'), '%Y-%m-%dT%H:%M:%S%z')
        except (ValueError, TypeError):
            self.date = None
        self.image = data.get('image')
        self.motds = [NewsMotd(item_data) for item_data in data.get('motds')] if data.get('motds') else None
        self.messages = [NewsMessage(item_data) for item_data in data.get('messages')] if data.get('messages') else None
        self.raw_data = data


class NewsMotd:
    """Represents News of a specific game mode.

    Attributes
    -----------
    id: :class:`str`
        The id of the motd.
    title: :class:`str`
        The title of the motd.
    body: Optional[:class:`str`]
        The body of the motd.
    image_url: :class:`str`
        The url of the image in 1920x1080.
    tile_image_url: :class:`str`
        The url of the tile image in 1024x512.
    hidden: :class:`bool`
        Whether the motd is hidden.
    spotlight: :class:`bool`
        Whether the motd is in spotlight.
    type: :class:`str`
        The type of the entry.
    raw_data: :class:`dict`
        The raw data from request. Can be used for saving and re-creating the class.
    """

    def __init__(self, data):
        self.id = data.get('id')
        self.title = data.get('title')
        self.body = data.get('body')
        self.image_url = data.get('image')
        self.tile_image_url = data.get('tileImage')
        self.sorting_priority = data.get('sortingPriority')
        self.raw_data = data


class NewsMessage:
    """Represents News of a specific game mode.

    Attributes
    -----------
    image_url: :class:`str`
        The url of the image.
    hidden: :class:`bool`
        Whether the entry is hidden.
    message_type: Optional[:class:`str`]
        The type of the message.
    type: :class:`str`
        The type of the entry.
    banner: :class:`str`
        The banner of the entry.
    title: :class:`str`
        The title of the entry.
    body: :class:`str`
        The body of the entry.
    spotlight: :class:`bool`
        Whether the entry is in spotlight.
    raw_data: :class:`dict`
        The raw data from request. Can be used for saving and re-creating the class.
    """

    def __init__(self, data):
        self.title = data.get('title')
        self.body = data.get('body')
        self.image_url = data.get('image')
        self.adspace = data.get('adspace')
        self.raw_data = data



import re

import setuptools
from setuptools import setup

version = ''
with open('fortnite_api/__init__.py') as f:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', f.read(), re.MULTILINE).group(1)

if not version:
    raise RuntimeError('version is not set')


readme = ''
with open('README.md') as f:
    readme = f.read()

setup(
    name='fortnite-api',
    author='Luc1412',
    author_email='Luc1412.lh@gmail.com',
    url='https://github.com/Fortnite-API/py-wrapper',
    project_urls={
        # "Documentation": "https://fortnite-api.readthedocs.io/en/latest/",
        "Issue tracker": "https://github.com/Fortnite-API/py-wrapper/issues",
        "Code": "https://github.com/Fortnite-API/py-wrapper"
    },
    version=version,
    packages=setuptools.find_packages(),
    license='MIT',
    description='A python wrapper for Fortnite-API.com',
    long_description=readme,
    long_description_content_type="text/markdown",
    install_requires=['requests>=2.22.0', 'aiohttp>3.8.6,<4'],
    python_requires='>=3.6.15',
    download_url=f'https://github.com/Fortnite-API/py-wrapper/archive/refs/tags/v{version}.tar.gz',
    keywords=['fortnite', 'fortnite-api.com', 'shop', 'cosmetics', 'fortnite api', 'fortnite shop'],
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Topic :: Internet',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
    ],
)



from datetime import datetime
from typing import List, Optional, Union

from .cosmetics import BrCosmetic, CarCosmetic, InstrumentCosmetic, JamTrack, LegoKitCosmetic
from .enums import ShopTileSize, CosmeticCompatibleMode


class Shop:

    def __init__(self, data: dict):
        self.hash: str = data['hash']
        self.date: datetime = datetime.strptime(data['date'], '%Y-%m-%dT%H:%M:%S%z')
        self.entries: List[ShopEntry] = [ShopEntry(entry) for entry in data['entries'] or []]
        self.raw_data: dict = data


class ShopEntry:

    def __init__(self, data: dict):
        self.regular_price: int = data['regularPrice']
        self.final_price: int = data['finalPrice']
        self.bundle: Optional[ShopBundle] = ShopBundle(data['bundle']) if data['bundle'] else None
        self.banner: Optional[ShopBanner] = ShopBanner(data['banner']) if data['banner'] else None
        self.giftable: bool = data['giftable']
        self.refundable: bool = data['refundable']
        self.sort_priority: int = data['sortPriority']
        self.layout_id: str = data['layoutId']
        self.layout: Optional[ShopLayout] = ShopLayout(data['layout']) if data['layout'] else None
        self.dev_name: str = data['devName']
        self.offer_id: str = data['offerId']
        self.display_asset_path: Optional[str] = data['displayAssetPath']
        try:
            self.tile_size: ShopTileSize = ShopTileSize(data['tileSize'])
        except ValueError:
            # This is a fix, because Epic believes that they should spell "normal" as "nomal"
            self.tile_size: ShopTileSize = ShopTileSize.NORMAL
        self.new_display_asset_path: Optional[str] = data['newDisplayAssetPath']
        self.new_display_asset: Optional[ShopNewDisplayAsset] = ShopNewDisplayAsset(data['newDisplayAsset']) if data[
            'newDisplayAsset'] else None
        self.items: List[Union[BrCosmetic, CarCosmetic, InstrumentCosmetic, LegoKitCosmetic, JamTrack]] = []
        if data['brItems']:
            self.items.extend([BrCosmetic(item) for item in data['brItems']])
        if data['cars']:
            self.items.extend([CarCosmetic(item) for item in data['cars']])
        if data['instruments']:
            self.items.extend([InstrumentCosmetic(item) for item in data['instruments']])
        if data['legoKits']:
            self.items.extend([LegoKitCosmetic(item) for item in data['legoKits']])
        if data['tracks']:
            self.items.extend([JamTrack(item) for item in data['tracks']])
        self.raw_data: dict = data


class ShopBundle:

    def __init__(self, data: dict):
        self.name: str = data['name']
        self.info: str = data['info']
        self.image_url: str = data['image']
        self.raw_data: dict = data


class ShopBanner:

    def __init__(self, data: dict):
        self.value: str = data['value']
        self.intensity: str = data['intensity']
        self.backend_value: str = data['backendValue']
        self.raw_data: dict = data


class ShopLayout:

    def __init__(self, data: dict):
        self.id: str = data['id']
        self.name: str = data['name']
        self.category: str = data['category']
        self.index: int = data['index']
        self.show_ineligible_offers: bool = data['showIneligibleOffers']
        self.background_url: str = data['background']
        self.raw_data: dict = data


class ShopNewDisplayAsset:

    def __init__(self, data: dict):
        self.id: str = data['id']
        self.cosmetic_id: Optional[str] = data['cosmeticId']
        self.material_instances: List[ShopMaterialInstance] = [ShopMaterialInstance(mi) for mi in
                                                               data['materialInstances']]
        self.raw_data: dict = data


class ShopMaterialInstance:

    def __init__(self, data: dict):
        self.id: str = data['id']
        self.primary_mode: CosmeticCompatibleMode = CosmeticCompatibleMode(data['primaryMode'])
        self.images: ShopMaterialInstanceImages = ShopMaterialInstanceImages(data['images'])
        self.colors: Optional[dict] = data['colors']
        self.scalings: Optional[dict] = data['scalings']
        self.flags: Optional[dict] = data['flags']
        self.raw_data: dict = data


class ShopMaterialInstanceImages:

    def __init__(self, data: dict):
        self.offer_image_url: str = data['OfferImage']
        self.fnm_texture_url: Optional[str] = data.get('FNMTexture')
        self.image_background_url: Optional[str] = data.get('ImageBackground')
        self.background_url: Optional[str] = data.get('Background')
        self.raw_data: dict = data


class BrShop:
    """Represents a Battle Royale shop.

    Attributes
    -----------
    hash: :class:`str`
        The hash of the shop.
    date: :class:`datetime.datetime`
        The timestamp of the .
    featured: Optional[List[:class:`BrShopEntry`]]
        A list of all featured entries.
    daily: Optional[List[:class:`BrShopEntry`]]
        A list of all daily entries.
    votes: Optional[List[:class:`BrShopEntry`]]
        A list of all vote entries.
    vote_winners: Optional[List[:class:`BrShopEntry`]]
        A list of all vote winner.
    raw_data: :class:`dict`
        The raw data from request. Can be used for saving and re-creating the class.
    """

    def __init__(self, data):
        self.hash = data.get('hash')
        try:
            self.date = datetime.strptime(data.get('date'), '%Y-%m-%dT%H:%M:%S%z')
        except (ValueError, TypeError):
            self.date = None
        self.featured = BrShopSection(data.get('featured')) if data.get('featured') else None
        self.daily = BrShopSection(data.get('daily')) if data.get('daily') else None
        self.special_featured = BrShopSection(data.get('specialFeatured')) if data.get('specialFeatured') else None
        self.special_daily = BrShopSection(data.get('specialDaily')) if data.get('specialDaily') else None
        self.votes = BrShopSection(data.get('votes')) if data.get('votes') else None
        self.vote_winners = BrShopSection(data.get('voteWinners')) if data.get('voteWinners') else None
        self.raw_data = data


class BrShopSection:

    def __init__(self, data):
        self.name = data.get('name')
        self.entries = [BrShopEntry(entry_data) for entry_data in data.get('entries')] if data.get('entries') else None
        self.raw_data = data


class BrShopEntry:
    """Represents a Battle Royale shop entry.

    Attributes
    -----------
    regular_price: :class:`int`
        The internal price.
    final_price: :class:`int`
        The price which is shown in-game.
    discount: :class:`int`
        The discount on the item.
    giftable: :class:`bool`
        Whether the item is giftable.
    refundable: :class:`bool`
        Whether the item is refundable.
    panel: :class:`int`
        The id of the panel in the featured section. -1 if the item is in no panel.
    sort_priority: :class:`int`
        The sort priority in the featured panels.
    banner: Optional[:class:`str`]
        The text of the banner. This text is shown in a arrow in-game.
    items: List[:class:`BrCosmetic`]
        A list of all cosmetics you get when you buy.
    raw_data: :class:`dict`
        The raw data from request. Can be used for saving and re-creating the class.
    """

    def __init__(self, data):
        self.regular_price = data.get('regularPrice')
        self.final_price = data.get('finalPrice')
        self.discount = self.regular_price - self.final_price if self.regular_price and self.final_price else None
        self.bundle = BrShopBundle(data.get('bundle')) if data.get('bundle') else None
        self.banner = BrShopBanner(data.get('banner')) if data.get('banner') else None
        self.giftable = data.get('giftable')
        self.refundable = data.get('refundable')
        self.sort_priority = data.get('sortPriority')
        self.categories = data.get('categories')
        self.section_id = data.get('sectionId')
        self.section = BrShopSectionNew(data.get('section')) if data.get('section') else None
        self.layout = BrShopLayout(data.get('layout')) if data.get('layout') else None
        self.dev_name = data.get('devName')
        self.offer_id = data.get('offerId')
        self.display_asset_path = data.get('displayAssetPath')
        self.tile_size = data.get('tileSize')
        self.new_display_asset_path = data.get('newDisplayAssetPath')
        self.new_display_asset = data.get('newDisplayAsset')
        self.items = [BrCosmetic(item_data) for item_data in data.get('items')]
        self.raw_data = data


class BrShopBundle:

    def __init__(self, data):
        self.name = data.get('name')
        self.info = data.get('info')
        self.image = data.get('image')
        self.raw_data = data


class BrShopBanner:

    def __init__(self, data):
        self.value = data.get('value')
        self.intensity = data.get('intensity')
        self.backend_value = data.get('backendValue')
        self.raw_data = data


class BrShopSectionNew:

    def __init__(self, data):
        self.id = data.get('id')
        self.name = data.get('name')
        self.index = data.get('index')
        self.landing_priority = data.get('landingPriority')
        self.sort_offers_by_ownership = data.get('sortOffersByOwnership')
        self.show_ineligible_offers = data.get('showIneligibleOffers')
        self.show_ineligible_offers_if_giftable = data.get('showIneligibleOffersIfGiftable')
        self.show_timer = data.get('showTimer')
        self.enable_toast_notification = data.get('enableToastNotification')
        self.hidden = data.get('hidden')


class BrShopLayout:

    def __init__(self, data):
        self.id = data.get('id')
        self.name = data.get('name')
        self.category = data.get('category')
        self.index = data.get('index')
        self.show_ineligible_offers = data.get('showIneligibleOffers')


class BrShopNewDisplayAsset:

    def __init__(self, data):
        self.id = data.get('id')
        self.material_instances = [BrShopMaterialInstance(mi) for mi in data.get('materialInstances')]


class BrShopMaterialInstance:

    def __init__(self, data):
        self.id = data.get('id')
        self.images = data.get('images')
        self.colors = data.get('colors')
        self.scalings = data.get('scalings')



class FortniteAPIException(Exception):
    pass


class NotFound(FortniteAPIException):
    pass


class Forbidden(FortniteAPIException):
    pass


class MissingSearchParameter(FortniteAPIException):
    pass


class MissingIDParameter(FortniteAPIException):
    pass


class ServiceUnavailable(FortniteAPIException):
    pass


class RateLimited(FortniteAPIException):
    pass


class Unauthorized(FortniteAPIException):
    pass
